﻿using System;
using System.Transactions;
using System.Data;
using System.Data.SqlClient;

namespace TransactionInfo
{
	class Program
	{
		static void Main(string[] args)
		{
			Console.WriteLine("Transaction Information Sample......");

			SimpleTest1();
			Console.WriteLine();
			SimpleTest2();
		}

		static void SimpleTest1()
		{
			TransactionScopeOption scopeOption = TransactionScopeOption.RequiresNew;
			TransactionOptions options = new TransactionOptions();
			options.IsolationLevel = System.Transactions.IsolationLevel.ReadCommitted;
			options.Timeout = TimeSpan.FromSeconds(120);

			Console.WriteLine("Simple Transaction Test #1......");
			using (TransactionScope scope = new TransactionScope(scopeOption, options)) {
				AccessDB();
				ShowCurrentTransactionInfo();

				// 새로운 트랜잭션(RequiresNew)이 시작했으므로 두 트랜잭션은 모두 로컬 트랜잭션이다.
				using(TransactionScope scope2 = new TransactionScope(TransactionScopeOption.RequiresNew)) {
					AccessDB();
					ShowCurrentTransactionInfo();
					scope2.Complete();
				}

				scope.Complete();
			}
			Console.WriteLine("Transaction end...");
		}

		static void SimpleTest2()
		{
			Console.WriteLine("Simple Transaction Test #2......");
			TransactionManager.DistributedTransactionStarted += delegate(object sender, TransactionEventArgs e) {
				Console.WriteLine("Distributed Transaction Started !! id = {0}", GetTransactionId(e.Transaction));
			};

			using (TransactionScope scope = new TransactionScope()) {
				AccessDB();
				ShowCurrentTransactionInfo();
				string txId1 = GetCurrentTransactionId();
				Transaction.Current.TransactionCompleted += delegate(object sender, TransactionEventArgs e) {
					Console.WriteLine("Transaction completed !! id={0}", txId1);
				};

				using (TransactionScope scope2 = new TransactionScope()) {
					ShowCurrentTransactionInfo();
					AccessDB();
					ShowCurrentTransactionInfo();
					string txId2 = GetCurrentTransactionId();
					Transaction.Current.TransactionCompleted += delegate(object sender, TransactionEventArgs e) {
						Console.WriteLine("Transaction completed !! id={0}", txId2);
					};
					scope2.Complete();
				}

				scope.Complete();
			}
			Console.WriteLine("Transaction end...");
		}

		static void AccessDB()
		{
			string connStr = "SERVER=(local);UID=Tester;PWD=test;DATABASE=TestDB";
			string query = "SELECT COUNT(*) FROM TestTable";
			SqlConnection conn = new SqlConnection(connStr);
			SqlCommand cmd = new SqlCommand(query, conn);

			conn.Open();
			int count = (int)cmd.ExecuteScalar();
			conn.Close();
			Console.WriteLine("  RecordCount = {0}", count);
		}

		static bool IsInTransaction()
		{
			return (Transaction.Current != null);
		}

		static bool IsDistributedTransaction(Transaction tx)
		{
			return (tx.TransactionInformation.DistributedIdentifier != Guid.Empty);
		}

		static string GetTransactionId(Transaction tx)
		{
			if (tx == null) {
				throw new InvalidOperationException("There is no transaction.");
			}
			if (IsDistributedTransaction(tx) == true) {
				return tx.TransactionInformation.DistributedIdentifier.ToString();
			}
			else {
				return tx.TransactionInformation.LocalIdentifier;
			}
		}

		static string GetCurrentTransactionId()
		{
			return GetTransactionId(Transaction.Current);
		}

		static void ShowTransactionInfo(Transaction tx)
		{
			TransactionInformation ti = tx.TransactionInformation;
			bool isDtx = IsDistributedTransaction(tx);
			string txId = GetTransactionId(tx);
			DateTime creationTime = ti.CreationTime;
			TransactionStatus status = ti.Status;
			
			Console.WriteLine("\tTransaction object type : {0}", tx.GetType().Name);
			Console.WriteLine("\tIsolation level : {0}", tx.IsolationLevel);
			Console.WriteLine("\tTransaction type : {0}", isDtx ? "Distributed" : "Local");
			Console.WriteLine("\tTransaction id : {0}", txId);
			Console.WriteLine("\tCreation time : {0}", creationTime);
			Console.WriteLine("\tStatus : {0}", status);
		}

		static void ShowCurrentTransactionInfo()
		{
			bool isInTx = IsInTransaction();

			Console.WriteLine("  >> Transaction Information:");
			Console.WriteLine("\tIs in transaction : {0}", isInTx);
			if (isInTx == true) {
				ShowTransactionInfo(Transaction.Current);
			}
		}
	}
}
